/* AMX Mod X script.
*   IOS Keeper Manager
*
*  Copyright (C) 2004 Pavol "PM" Marko
*
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*
*  Version 0.1
*
*/

/* used entities:
 trigger_PenaltyBox
 trigger_SideLine
 trigger_GoalLine
 info_ball_start 
*/

#include <amxmodx> 
#include <amxmisc>
#include <engine>

/********** plugin info **********/
new PLINFO_NAME[] = "KeeperMngr";
new PLINFO_VERSION[] = "0.1";
new PLINFO_AUTHORS[] = "PM";

// RC Interface version
#define PLINFO_RCIFVERS 1

/********** config **********/

// required admin level
#define ADMINLEVEL ADMIN_LEVEL_H

/********** limit levels **********/
// 1
#define LL1_NONE 0		/* unlimited */
#define LL1_HALF 1		/* line parallel to trigger_GoalLine going through info_ball_start */
#define LL1_PENBOX 2	/* keeper's penalty box */
#define LL1_SPAWN 3		/* keeper's spawn */

// 2
#define LL2_NONE 0		/* unlimited */
#define LL2_SIDELINE 1	/* limited to side lines */
#define LL2_PENBOX 2	/* limited to penbox */
#define LL2_SPAWN 3		/* keeper's spawn */

/********** plane constants **********/
// normal
#define PLANE_A 0
#define PLANE_B 1
#define PLANE_C 2
// plane shift distance
#define PLANE_D 3

/********** flags **********/
#define PLFL_MAKEKEEPER 1
#define PLFL_MAKEUNKEEPER 2

/********** globals **********/
new g_LimitLevel1;						// Limit level 1
new g_LimitLevel2;						// Limit level 2

new g_Keepers[2];						// Both keepers

new bool:g_ModSupported;				// are we on ios?
new bool:g_Enabled;						// can't be switched by user in this version

new Float:g_MidPlane[4];				// plane parallel to trigger_goal going through info_ball_start
new Float:g_PenboxPlane[2][4];			// penalty box normal plane for both teams
new Float:g_SpawnPlane[2][4];			// spawn normal plane for both teams

new Float:g_SidePlane[2][4];			// both side planes
new Float:g_PenboxSidePlane[2][4];		// both side planes of the penboxes
new Float:g_SpawnSidePlane[4];			// spawn side plane

new g_PlayerFlags[32];					// flags

/********** utils **********/
stock util_setdefaults()
{
	g_LimitLevel1 = LL1_HALF;
	g_LimitLevel2 = LL2_NONE;
}

stock util_concmd_print(id, text[])
{
	if (id > 0)
		client_print(id, print_console, text);
	else
		server_print(text);
}

stock util_readintarg(argnum)
{
	new tmp[32];
	read_argv(argnum, tmp, 31);
	return str_to_num(tmp);
}

/********** actual stuff :) **********/
stock vec_normalize(Float:vec[])
{
	new Float:invlen = 1/floatsqroot(vec[0]*vec[0] + vec[1]*vec[1] + vec[2]*vec[2]);
	vec[0] *= invlen;
	vec[1] *= invlen;
	vec[2] *= invlen;
}

stock vec_cross(Float:vec1[], Float:vec2[], Float:out[])
{
	out[0] = vec1[1]*vec2[2] - vec1[2]*vec2[1];
	out[1] = vec1[2]*vec2[0] - vec1[0]*vec2[2];
	out[2] = vec1[0]*vec2[1] - vec1[1]*vec2[0];
}

stock Float:vec_dot(Float:vec1[], Float:vec2[])
{
	return vec1[0]*vec2[0] + vec1[1]*vec2[1] + vec1[2]*vec2[2];
}

stock vec_neg(Float:vec[], Float:out[])
{
	out[0] = -vec[0];
	out[1] = -vec[1];
	out[2] = -vec[2];
}

stock vec_copy(Float:vecIn[], Float:vecOut[])
{
	vecOut[0] = vecIn[0];
	vecOut[1] = vecIn[1];
	vecOut[2] = vecIn[2];
}

stock plane_3p(Float:p1[], Float:p2[], Float:p3[], Float:plane[])
{
	new Float:normalA[3], Float:normalB[3];
	// normalA = Normalize(p3 - p1);
	normalA[0] = p3[0] - p1[0];
	normalA[1] = p3[1] - p1[1];
	normalA[2] = p3[2] - p1[2];
	vec_normalize(normalA);
	
	// normalB = Normalize(p3 - p2);
	normalB[0] = p3[0] - p2[0];
	normalB[1] = p3[1] - p2[1];
	normalB[2] = p3[2] - p2[2];
	vec_normalize(normalB);
	
	// plane normal = Normalize(normalA cross normalB)
	vec_cross(normalA, normalB, plane);
	vec_normalize(plane);
	
	// plane shift distance = (-p1) dot plane normal
	new Float:tmp[3];
	vec_neg(plane, tmp);
	plane[PLANE_D] = vec_dot(tmp, p1);
	
}

stock Float:plane_dst2point(Float:plane[], Float:point[])
{
	// return normal dot point + D
	return vec_dot(plane, point) + plane[PLANE_D];
}

stock plane_copy(Float:planeIn[], Float:planeOut[])
{
	planeOut[0] = planeIn[0];
	planeOut[1] = planeIn[1];
	planeOut[2] = planeIn[2];
	planeOut[3] = planeIn[3];
}

// Find out everything important about this map and store to globals
gather_mapinfo()
{
	new tmpEnt;
	new Float:tmpVec1[3];
	new Float:tmpVec2[3];
	new Float:tmpVec3[3];
	new Float:tmpScalar;
	
	// ***** make the midplane *****
	// first, construct a plane out of 3 known points: info_team1_corner0, info_team1_corner1 and info_team1_corner0 with z + 20
	// point1
	tmpEnt = find_ent_by_class(0, "info_team1_corner0");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_corner0 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	
	// point2
	tmpEnt = find_ent_by_class(0, "info_team1_corner1");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_corner1 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec2);
	
	// point3
	tmpVec3[0] = tmpVec2[0];
	tmpVec3[1] = tmpVec2[1];
	tmpVec3[2] = tmpVec2[2] + 1000.0;
	
	plane_3p(tmpVec1, tmpVec2, tmpVec3, g_MidPlane);
	
	// make sure the plane normal points to team0 side
	if (plane_dst2point(g_MidPlane, tmpVec1) < 0.0)
	{
		// negate normal and dist
		g_MidPlane[PLANE_A] = -g_MidPlane[PLANE_A];
		g_MidPlane[PLANE_B] = -g_MidPlane[PLANE_B];
		g_MidPlane[PLANE_C] = -g_MidPlane[PLANE_C];
		g_MidPlane[PLANE_D] = -g_MidPlane[PLANE_D];
	}
	
	// now move the plane to the ball start point: subtract the distance between the plane and the point from PLANE_D
	tmpEnt = find_ent_by_class(0, "info_ball_start");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_ball_start in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	
	g_MidPlane[PLANE_D] -= plane_dst2point(g_MidPlane, tmpVec1);
	
	// ***** make the penbox normal planes *****
	// parallel plane to the midplane.
	tmpEnt = find_ent_by_class(0, "trigger_PenaltyBox");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find trigger_PenaltyBox in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_mins, tmpVec1);
	entity_get_vector(tmpEnt, EV_VEC_maxs, tmpVec2);
	
	if (floatabs(plane_dst2point(g_MidPlane, tmpVec2)) < floatabs(plane_dst2point(g_MidPlane, tmpVec1)))
	{
		// tmpVec1 = tmpVec2
		vec_copy(tmpVec2, tmpVec1);
	}
	
	// g_PenboxPlane[0] = g_MidPlane
	// g_PenboxPlane[1] = g_MidPlane
	plane_copy(g_MidPlane, g_PenboxPlane[0]);
	plane_copy(g_MidPlane, g_PenboxPlane[1]);
	
	g_PenboxPlane[0][PLANE_D] -= plane_dst2point(g_MidPlane, tmpVec1);
	g_PenboxPlane[1][PLANE_D] += plane_dst2point(g_MidPlane, tmpVec1);
	
	// ***** make the spawn normal plane *****
	// parallel plane to the midplane.
	tmpEnt = find_ent_by_class(0, "info_team1_player1");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_player1 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	
	// g_SpawnPlane[0] = g_MidPlane
	// g_SpawnPlane[0] = g_MidPlane
	plane_copy(g_MidPlane, g_SpawnPlane[0]);
	plane_copy(g_MidPlane, g_SpawnPlane[1]);
	
	g_SpawnPlane[0][PLANE_D] -= plane_dst2point(g_MidPlane, tmpVec1);
	g_SpawnPlane[1][PLANE_D] += plane_dst2point(g_MidPlane, tmpVec1);
	
	// ***** make the spawn sideplane *****
	// make it out of three points: keeper1 spawn, keeper2 spawn, keeper2 spawn with z + 20
	
	tmpEnt = find_ent_by_class(0, "info_team1_player1");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_player1 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	
	// vec2: vec1 on midplane
	tmpScalar = plane_dst2point(g_MidPlane, tmpVec1);
	tmpVec2[0] = tmpVec1[0] - g_MidPlane[0]*tmpScalar;
	tmpVec2[1] = tmpVec1[1] - g_MidPlane[1]*tmpScalar;
	tmpVec2[2] = tmpVec1[2] - g_MidPlane[2]*tmpScalar;
	
	tmpVec3[0] = tmpVec2[0];
	tmpVec3[1] = tmpVec2[1];
	tmpVec3[2] = tmpVec2[2] + 1000.0;
	
	plane_3p(tmpVec1, tmpVec2, tmpVec3, g_SpawnSidePlane);
	
	// ***** make the sideplanes *****
	// parallel to the spawn sideplane, goes through corner0 and corner1
	
	tmpEnt = find_ent_by_class(0, "info_team1_corner0");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_corner0 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	plane_copy(g_SpawnSidePlane, g_SidePlane[0]);
	g_SidePlane[0][PLANE_D] -= plane_dst2point(g_SpawnSidePlane, tmpVec1);
	
	tmpEnt = find_ent_by_class(0, "info_team1_corner1");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_corner0 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	plane_copy(g_SpawnSidePlane, g_SidePlane[1]);
	g_SidePlane[1][PLANE_D] -= plane_dst2point(g_SpawnSidePlane, tmpVec1);
	
	// make sure their normals point to the ball start
	tmpEnt = find_ent_by_class(0, "info_ball_start");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find info_team1_corner0 in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_origin, tmpVec1);
	
	if (plane_dst2point(g_SidePlane[0], tmpVec1) < 0.0)
	{
		g_SidePlane[0][0] = -g_SidePlane[0][0];
		g_SidePlane[0][1] = -g_SidePlane[0][1];
		g_SidePlane[0][2] = -g_SidePlane[0][2];
		g_SidePlane[0][3] = -g_SidePlane[0][3];
	}
	if (plane_dst2point(g_SidePlane[1], tmpVec1) < 0.0)
	{
		g_SidePlane[1][0] = -g_SidePlane[1][0];
		g_SidePlane[1][1] = -g_SidePlane[1][1];
		g_SidePlane[1][2] = -g_SidePlane[1][2];
		g_SidePlane[1][3] = -g_SidePlane[1][3];
	}
	
	// ***** make the penbox sideplanes *****
	// parallel to sideplanes, penboxsideplane0 is near to sideplane0 and penboxsideplan1 is near to sideplane1
	
	tmpEnt = find_ent_by_class(0, "trigger_PenaltyBox");
	if (tmpEnt <= 0)
	{
		log_amx("Couldn't find trigger_PenaltyBox in map!");
		g_Enabled = false;
		return;
	}
	entity_get_vector(tmpEnt, EV_VEC_mins, tmpVec1);
	entity_get_vector(tmpEnt, EV_VEC_maxs, tmpVec2);
	
	// g_PenboxSidePlane[0] = g_SidePlane[0]
	// g_PenboxSidePlane[1] = g_SidePlane[1]
	plane_copy(g_SidePlane[0], g_PenboxSidePlane[0]);
	plane_copy(g_SidePlane[1], g_PenboxSidePlane[1]);
	
	if (floatabs(plane_dst2point(g_SidePlane[0], tmpVec1)) < floatabs(plane_dst2point(g_SidePlane[0], tmpVec2)))
	{
		g_PenboxSidePlane[0][PLANE_D] -= plane_dst2point(g_PenboxSidePlane[0], tmpVec1);
		g_PenboxSidePlane[1][PLANE_D] -= plane_dst2point(g_PenboxSidePlane[1], tmpVec2);
	}
	else
	{
		g_PenboxSidePlane[0][PLANE_D] -= plane_dst2point(g_PenboxSidePlane[1], tmpVec1);
		g_PenboxSidePlane[1][PLANE_D] -= plane_dst2point(g_PenboxSidePlane[0], tmpVec2);
	}
}

// Do the actual keeper limiting
public limit_keepers()
{
	if (!g_Enabled)
		return;
	
	new i;
	new Float:tmpVec1[3];
	new Float:tmpScalar;
	for (i = 0; i < 2; ++i)
	{
		// limit1
		if (g_Keepers[i])
		{
			if (g_LimitLevel1 == LL1_HALF)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				tmpScalar = plane_dst2point(g_MidPlane, tmpVec1);
				
				if ((i && tmpScalar < 0) || (!i && tmpScalar > 0))
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_MidPlane[0] * tmpScalar;
					tmpVec1[1] -= g_MidPlane[1] * tmpScalar;
					tmpVec1[2] -= g_MidPlane[2] * tmpScalar;
					entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				}
			}
			else if (g_LimitLevel1 == LL1_PENBOX)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				tmpScalar = plane_dst2point(g_PenboxPlane[i], tmpVec1);
				
				if ((i && tmpScalar < 0) || (!i && tmpScalar > 0))
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_PenboxPlane[i][0] * tmpScalar;
					tmpVec1[1] -= g_PenboxPlane[i][1] * tmpScalar;
					tmpVec1[2] -= g_PenboxPlane[i][2] * tmpScalar;
					entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				}
			}
			else if (g_LimitLevel1 == LL1_SPAWN)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				tmpScalar = plane_dst2point(g_SpawnPlane[i], tmpVec1);
				
				// move him to the plane
				// origin = origin - (plane normal * distance);
				tmpVec1[0] -= g_SpawnPlane[i][0] * tmpScalar;
				tmpVec1[1] -= g_SpawnPlane[i][1] * tmpScalar;
				tmpVec1[2] -= g_SpawnPlane[i][2] * tmpScalar;
				entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
			}
			// limit2
			if (g_LimitLevel2 == LL2_SIDELINE)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				
				tmpScalar = plane_dst2point(g_SidePlane[0], tmpVec1);
				if (tmpScalar < 0.0)
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_SidePlane[0][0] * tmpScalar;
					tmpVec1[1] -= g_SidePlane[0][1] * tmpScalar;
					tmpVec1[2] -= g_SidePlane[0][2] * tmpScalar;
					
				}
				tmpScalar = plane_dst2point(g_SidePlane[1], tmpVec1);
				if (tmpScalar < 0.0)
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_SidePlane[1][0] * tmpScalar;
					tmpVec1[1] -= g_SidePlane[1][1] * tmpScalar;
					tmpVec1[2] -= g_SidePlane[1][2] * tmpScalar;
					
				}
				
				entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
			}
			else if (g_LimitLevel2 == LL2_PENBOX)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				
				tmpScalar = plane_dst2point(g_PenboxSidePlane[0], tmpVec1);
				if (tmpScalar < 0.0)
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_PenboxSidePlane[0][0] * tmpScalar;
					tmpVec1[1] -= g_PenboxSidePlane[0][1] * tmpScalar;
					tmpVec1[2] -= g_PenboxSidePlane[0][2] * tmpScalar;
					
				}
				tmpScalar = plane_dst2point(g_PenboxSidePlane[1], tmpVec1);
				if (tmpScalar < 0.0)
				{
					// move him to the plane
					// origin = origin - (plane normal * distance);
					tmpVec1[0] -= g_PenboxSidePlane[1][0] * tmpScalar;
					tmpVec1[1] -= g_PenboxSidePlane[1][1] * tmpScalar;
					tmpVec1[2] -= g_PenboxSidePlane[1][2] * tmpScalar;
					
				}
				
				entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
			}
			else if (g_LimitLevel2 == LL2_SPAWN)
			{
				entity_get_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
				tmpScalar = plane_dst2point(g_SpawnSidePlane, tmpVec1);
				// move him to the plane
				// origin = origin - (plane normal * distance);
				tmpVec1[0] -= g_SpawnSidePlane[0] * tmpScalar;
				tmpVec1[1] -= g_SpawnSidePlane[1] * tmpScalar;
				tmpVec1[2] -= g_SpawnSidePlane[2] * tmpScalar;
				entity_set_vector(g_Keepers[i], EV_VEC_origin, tmpVec1);
			}
		}
	}
}

public icmd_keeper(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED;
		
	if (g_Keepers[0] && g_Keepers[1])
	{
		util_concmd_print(id, "Both teams already have keepers!");
	}
	
	new targ[32];
	read_argv(1, targ, 31);
	
	new player = cmd_target(id, targ, 1 | 2 | 8);
	if (!player)
		return PLUGIN_HANDLED;
	
	client_cmd(player, "changeteam");
	g_PlayerFlags[player] |= PLFL_MAKEKEEPER;
	g_PlayerFlags[player] &= ~PLFL_MAKEUNKEEPER;

	return PLUGIN_HANDLED;
}

public icmd_unkeeper(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED;
	new targ[32];
	read_argv(1, targ, 31);
	
	new player = cmd_target(id, targ, 1 | 2 | 8);
	if (!player)
		return PLUGIN_HANDLED;
	
	client_cmd(player, "changeteam");
	
	g_PlayerFlags[player] |= PLFL_MAKEUNKEEPER;
	g_PlayerFlags[player] &= ~PLFL_MAKEKEEPER;
	
	return PLUGIN_HANDLED;
}

public ev_selectteam(id)
{
	if (g_PlayerFlags[id] & PLFL_MAKEKEEPER)
	{
		if (g_Keepers[0])
			client_cmd(id, "slot3");
		else
			client_cmd(id, "slot2");
	}
	else if (g_PlayerFlags[id] & PLFL_MAKEUNKEEPER)
	{
		client_cmd(id, "slot6");		// Auto assign
	}
}

public ev_selectpos(id)
{
	if (g_PlayerFlags[id] & PLFL_MAKEKEEPER)
	{
		client_cmd(id, "slot1");
	}
	if (g_PlayerFlags[id] & PLFL_MAKEUNKEEPER)
	{
		client_cmd(id, "slot10");
		client_cmd(id, "slot9");
		client_cmd(id, "slot8");
		client_cmd(id, "slot7");
		client_cmd(id, "slot6");
		client_cmd(id, "slot5");
		client_cmd(id, "slot4");
		client_cmd(id, "slot3");
		client_cmd(id, "slot2");
	}
	g_PlayerFlags[id] &= ~(PLFL_MAKEKEEPER | PLFL_MAKEUNKEEPER);
}

/********** plugin init & commands **********/
public plugin_init()
{
	register_plugin(PLINFO_NAME, PLINFO_VERSION, PLINFO_AUTHORS);
	register_concmd("amx_keeperlimit", "icmd_setlimit", ADMINLEVEL, " <ll1> <ll2> (both 0-3)");
	register_concmd("amx_keeper", "icmd_keeper", ADMINLEVEL, " <name|#id> : Makes a player join keeper pos");
	register_concmd("amx_unkeeper", "icmd_unkeeper", ADMINLEVEL, " <name|#id> : Makes a player join normal pos");
	
	register_event("VGUIMenu", "ev_selectteam", "b", "1=2");
	register_event("VGUIMenu", "ev_selectpos", "b", "1=10", "1=9");
	
	set_task(0.1, "limit_keepers", 0, "", 0, "b");
	
	register_logevent("join_keeper", 3, "1=changed role to");
	
	new modname[32];
	get_modname(modname, 31);
	g_ModSupported = equali(modname, "ios") ? true : false;
	
	if (!g_ModSupported)
	{
		g_Enabled = false;
		log_amx("This plugin only runs on IOS!");
		return;
	}
	else
	{
		g_Enabled = true;
	}
	
	gather_mapinfo();
	util_setdefaults();
}

public icmd_setlimit(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED;

	if (!g_ModSupported)
	{
		util_concmd_print(id, " KeeperMngr only runs on IOS!");
		return PLUGIN_HANDLED;
	}

	if (!g_Enabled)
	{
		util_concmd_print(id, " KeeperMngr is disabled!");
		return PLUGIN_HANDLED;
	}
	
	new ll1 = util_readintarg(1);
	new ll2 = util_readintarg(2);
	new bool:valid = true;
	if (ll1 < 0 || ll1 > 3)
	{
		util_concmd_print(id, " Valid values for ll1: 0 (none), 1 (half), 2 (penbox), 3 (spawn)");
		valid = false;
	}
	if (ll2 < 0 || ll2 > 3)
	{
		util_concmd_print(id, " Valid values for ll2: 0 (none), 1 (sideline), 2 (penbox), 4 (spawn)");
		valid = false;
	}
	if (!valid)
		return PLUGIN_HANDLED;
	
	g_LimitLevel1 = ll1;
	g_LimitLevel2 = ll2;
	
	return PLUGIN_HANDLED;
}

public join_keeper()
{
	new user[128];
	new userId;
	read_logargv(0, user, 127);
	parse_loguser(user, "", 0, userId);
	
	g_PlayerFlags[userId] &= ~(PLFL_MAKEKEEPER | PLFL_MAKEUNKEEPER);
	
	if (userId == g_Keepers[0])
		g_Keepers[0] = 0;
	if (userId == g_Keepers[1])
		g_Keepers[1] = 0;

	new role[8];
	read_logargv(2, role, 7);
	
	if (str_to_num(role) != 1)		// role 1 = keeper
		return;
	
	// seems like team is not available sometimes. Wait 1 sec
	new params[1];
	params[0] = userId;
	set_task(1.0, "join_keeper_finish", 0, params, 1);
}

public join_keeper_finish(params[1])
{
	new team = entity_get_int(params[0], EV_INT_team);
	if (team < 0)
		team = 0;
	if (team > 1)
		team = 1;
	g_Keepers[team] = params[0];
}

public client_disconnect(id)
{
	if (id == g_Keepers[0])
		g_Keepers[0] = 0;
	if (id == g_Keepers[1])
		g_Keepers[1] = 0;
		
	g_PlayerFlags[id] = 0;
}